#version 430
#extension GL_ARB_separate_shader_objects : enable










struct SComputeInput
{
	uvec3 GlobalThreadID; uvec3 LocalThreadID; uvec3 GroupID;};

layout( std140) uniform UData
{
	uint Face;
	uint MipLevel;
	uint Size;
	uint Padding;
} Data;
 uniform samplerCube InputCubeTexture;
 uniform writeonly image2D OutputTexture;





void ComputeMain( in SComputeInput  cInput  )
{

	uint face =  Data.Face ;
	uint size =  Data.Size ;
	uint mipLevel =  Data.MipLevel ;
	uint padding =  Data.Padding ;

	if (cInput.GlobalThreadID.x >= (size + padding * 2) || cInput.GlobalThreadID.y >= (size + padding * 2)) return;

	vec2 totalSize =  vec2(size + padding * 2) ;
	vec2 uv = ( vec2(cInput.GlobalThreadID.xy)  - (totalSize / 2) + vec2(.5f, .5f)) /  float(size) ;
	uv *= 2.0f;
	uv *= -1.;
	vec2 dUv = max(abs(uv) - vec2(1,1), vec2(0., 0.));
	uv = clamp(uv, vec2(-1,-1), vec2(1,1));
	float d = max(dUv.x, dUv.y);
	vec3 DArr[6] =
	{
		vec3(1-d, uv.y, uv.x),
		vec3(-1+d, uv.y, -uv.x),
		vec3(-uv.x, 1-d, -uv.y),
		vec3(-uv.x, -1+d, uv.y),
		vec3(-uv.x, uv.y, 1-d),
		vec3(uv.x, uv.y, -1+d)
	};

	vec3 D = DArr[face];
	D = normalize(DArr[face]);

	vec3 col =  textureLod(InputCubeTexture, D, max(0.0f, float(mipLevel) - 1.0f)) .rgb;
	if (dUv.x > 0.0f && dUv.y > 0.0f)
		col = vec3(0., 0., 0.);
	imageStore(OutputTexture, ivec2(cInput.GlobalThreadID.xy), vec4(col, 1.0f)) ;
}


layout(local_size_x = 32, local_size_y = 32, local_size_z = 1) in;
void main()
{
	SComputeInput cInput;
	cInput.GlobalThreadID = gl_GlobalInvocationID;
	cInput.LocalThreadID = gl_LocalInvocationID;
	cInput.GroupID = gl_WorkGroupID;
	ComputeMain(cInput);
}
